@@ -148,6 +148,7 @@ _fill_app_list_liststore_from_app_config_path(GtkListStore* list_store)
   }
 }
 
+/*
 static gboolean 
 search_app_func(GtkTreeModel *model,
 		gint column,
@@ -171,7 +172,7 @@ search_app_func(GtkTreeModel *model,
   }
   return TRUE;
 }
-
+*/
 static void
 _call_application(GtkTreeView       *tree_view,
 		  GtkTreePath       *path,
@@ -229,16 +230,49 @@ _call_application(GtkTreeView       *tree_view,
   }
   g_object_unref(proxy);
 }
-/*
-static gint
-sort_app_name_ignore_case(GtkTreeModel *model,
-			  GtkTreeIter *a,
-			  GtkTreeIter *b,
-			  gpointer user_data)
+
+static gboolean
+_filter_app_list_func(GtkTreeModel* model,
+		      GtkTreeIter* iter,
+		      gpointer data)
 {
+  g_return_if_fail(GTK_IS_ENTRY(data));
+  GtkEntry* entry = GTK_ENTRY(data);
   
+  if(gtk_entry_get_text_length(entry)>0)
+  {
+    const gchar* text = gtk_entry_get_text(entry);
+    gchar* app_name = NULL;
+    
+    gtk_tree_model_get(model, iter,
+		       1, &app_name,
+		       -1);
+    gboolean visible = FALSE;
+    if(app_name)
+    {
+      gchar* app_name_downcase = g_utf8_strdown(app_name, -1);
+      if(strstr(app_name_downcase, text)!=NULL)
+      {
+	visible = TRUE;
+      }
+      g_free(app_name);
+      g_free(app_name_downcase);
+    }
+    return visible;
+  }
+  
+  return TRUE;
 }
-*/
+
+static void
+_refilter_by_text(GtkEditable *entry,
+		  gpointer     user_data)
+{
+  g_return_if_fail(GTK_IS_TREE_MODEL_FILTER(user_data));
+  GtkTreeModelFilter* filter = GTK_TREE_MODEL_FILTER(user_data);
+  gtk_tree_model_filter_refilter(filter);
+}
+
 static gboolean
 _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* desktop_plugin)
 {
@@ -246,6 +280,8 @@ _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* d
   GtkWidget* search_bar;
   GtkWidget *app_selector;
   GtkListStore* app_list;
+  GtkTreeModel* filtered_app_list;
+
   dialog = gtk_dialog_new();/*_with_buttons("Start",
 				       NULL,
 				       (GtkDialogFlags)0,
@@ -261,12 +297,15 @@ _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* d
   app_list = gtk_list_store_new(3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
   gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(app_list),
 				       1, GTK_SORT_ASCENDING);
-/*
-  gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(app_list),
-				  sort_app_name_ignore_case,
-				  NULL, NULL);
-*/
   _fill_app_list_liststore_from_app_config_path(app_list);
+  
+  filtered_app_list = gtk_tree_model_filter_new(GTK_TREE_MODEL(app_list),
+						NULL);
+  gtk_tree_model_filter_set_visible_func(GTK_TREE_MODEL_FILTER(filtered_app_list),
+					 _filter_app_list_func,
+					 search_bar,
+					 NULL);
+
   GtkTreeViewColumn* column_icon = gtk_tree_view_column_new();
   GtkTreeViewColumn* column_app_name = gtk_tree_view_column_new();
   GtkCellRenderer* icon_renderer = gtk_cell_renderer_pixbuf_new();
@@ -290,28 +329,32 @@ _button_release(GtkWidget* widget, GdkEventButton *event, AppSearchHomePlugin* d
 				      NULL);
 
   gtk_tree_view_set_model(GTK_TREE_VIEW(app_selector),
-			  GTK_TREE_MODEL(app_list));
+			  GTK_TREE_MODEL(filtered_app_list));
   
-  g_object_unref(app_list);
+
+  /*
   gtk_tree_view_set_search_equal_func(GTK_TREE_VIEW(app_selector),
 				      search_app_func,
 				      desktop_plugin,
 				      NULL);
-
+  */
+  /*
   gtk_tree_view_set_search_entry(GTK_TREE_VIEW(app_selector),
 				 GTK_ENTRY(search_bar));
-
+  */
   GtkWidget* pan = hildon_pannable_area_new();
   gtk_container_add(GTK_CONTAINER(pan), app_selector);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), pan, TRUE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox), search_bar,FALSE,FALSE,0); 
   g_signal_connect(app_selector, "row-activated", G_CALLBACK(_call_application), dialog);
-
-  gtk_window_set_default_size(GTK_WINDOW(dialog),-1,350);
+  g_signal_connect(search_bar, "changed", G_CALLBACK(_refilter_by_text), filtered_app_list);
+  gtk_window_set_default_size(GTK_WINDOW(dialog),-1,250);
   gtk_widget_grab_focus(search_bar);
   gtk_widget_show_all(dialog);
   gtk_dialog_run(GTK_DIALOG(dialog));
   gtk_widget_destroy(dialog);
+  g_object_unref(app_list);
+  g_object_unref(filtered_app_list);
 }
 
 
